--------------------------------------------------------------------------------
--
-- Test top-level for PowerSequencer code. Targetted for MAX10 Eval Board
-- Mark Frost, Altera UK, March 2016 (mfrost@altera.com)
--
-- QSYS generates the system and VHDL (for instantiation below) plus
-- the .sopcinfo file which the Eclipse tools need
-- Pretty much everything happens in the NIOS code - here we just
-- instantiate the QSYS system and wire it up to the IO.
-- We also instantiate a simple LED flasher, driven by the QSys PLL output 
-- as a FPGA Is Alive indicator (LED D5)
--
-- Future work would be to add in PWM and I2C modules for enhanced control of
-- power modules.
--------------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

--============================================================================--
--================================== ENTITY ==================================--
--============================================================================--
-- Refer to JD's schematic to understand the relevance of these signals

entity PowerSeq1 is
	port (
		-----------------------------------------
		-- Default MAX10 signals, reset and clock 
		-----------------------------------------
		Reset_N  		: in  std_logic;                      
		Clk      		: in  std_logic;
		
		--------------------------------------------
		-- Power Supply input control and monitoring
		--------------------------------------------
		-- Supply_ADC_In - this is one of the fixed ADC inputs and does not need defining here

		---------------------------------------------------------
		-- Control & Monitoring signals for Power Supply Group #1
		---------------------------------------------------------
		Enable_Group_1	: out std_logic;
		POK_Group_1		: in  std_logic;
		Discharge_1		: out std_logic;
		-- Vout 1 - one of the fixed ADC inputs and does not need defining here
		
		---------------------------------------------------------
		-- Control & Monitoring signals for Power Supply Group #2
		---------------------------------------------------------
		Enable_Group_2	: out std_logic;
		POK_Group_2		: in  std_logic;
		Discharge_2		: out std_logic;
		-- Vout 2 - one of the fixed ADC inputs and does not need defining here

		---------------------------------------------------------
		-- Control & Monitoring signals for Power Supply Group #3
		---------------------------------------------------------
		Enable_Group_3	: out std_logic;
		POK_Group_3		: in  std_logic;
		Discharge_3		: out std_logic;
		-- Vout 3 - one of the fixed ADC inputs and does not need defining here

		---------------------------------------------------------
		-- Control & Monitoring signals for Power Supply Group #4
		---------------------------------------------------------
		Enable_Group_4	: out std_logic;
		POK_Group_4		: in  std_logic;
		Discharge_4		: out std_logic;
		-- Vout 4 - one of the fixed ADC inputs and does not need defining here
		
		-------------------------------------------------
		-- Eval board's 5 user LEDs and test DIP switches
		-------------------------------------------------
		User_Switch_1	: in  std_logic;
		User_Switch_2	: in  std_logic;
		User_Switch_3	: in  std_logic;
		User_Switch_4	: in  std_logic;
		User_Switch_5	: in  std_logic;
		LEDs_Go  		: out std_logic_vector(4 downto 0)   
		);
end PowerSeq1;

--============================================================================--
--================================ ARCHITECTURE ==============================--
--============================================================================--
architecture RTL of PowerSeq1 is

	------------------------------------------------------------------------------
	------------------------------------------------------------------------------
	-- COMPONENT DECLARATIONS
	------------------------------------------------------------------------------
	------------------------------------------------------------------------------

  	--------------------------------------------------------------------------
	-- Instantiate NIOS QSYS generated component
	-- Component and port mapping declarations copied from auto-generated code
	--------------------------------------------------------------------------
	component power1 is
		port (
			altpll_0_c1_clk                : out   std_logic;                           -- clk
			clk_clk                        : in    std_logic                    := 'X'; -- clk
			reset_reset_n                  : in    std_logic                    := 'X'; -- reset_n
			test_pio_output_in_port        : in    std_logic_vector(15 downto 0) := (others => 'X'); -- in_port
			test_pio_output_out_port       : out   std_logic_vector(15 downto 0)                     -- out_port
		);
	end component power1;

	--------------------------------------------------------------------------
	-- RTL based LED flasher, just because....
	--------------------------------------------------------------------------
	component led_flash
		port (
			Res_N         : in  std_logic;                      
			Clk           : in  std_logic; 
			LED_drive     : out std_logic_vector(3 downto 0)                       
			);
	end component;

	------------------------------------------------------------------------------
	------------------------------------------------------------------------------
	-- SIGNAL DECLARATIONS
	------------------------------------------------------------------------------
	------------------------------------------------------------------------------
	signal NIOS_Input_Internal : std_logic_vector(15 downto 0); 
	signal NIOS_Output_Internal : std_logic_vector(15 downto 0); 
	signal int_PLL_clk : std_logic;	
	signal LedFlash_FPGA_Alive : std_logic_vector(3 downto 0);
  
	begin

	------------------------------------
	-- Instantiation of main QSys system
	------------------------------------
	u0_main : component power1
		port map (
			clk_clk       => Clk,        
			reset_reset_n => Reset_N,   
			test_pio_output_out_port => NIOS_Output_Internal,  					
			test_pio_output_in_port => NIOS_Input_Internal,  					
			altpll_0_c1_clk => int_PLL_clk         								-- Export PLL output clock (80MHz)
		);
		
	--------------------------------------------------------------------------------------------
	-- Hardware LED HeartBeat. Driven by QSys defined PLL output so a good check of the FPGA
	--------------------------------------------------------------------------------------------
	u1_keep_alive_led: led_flash                                         
		port map (
			Res_N 		=> Reset_N,
			Clk        	=> int_PLL_clk,
			LED_drive  	=> LedFlash_FPGA_Alive
		);
		
	------------------------------------------------------------------------------
	-- IO assignments.
	-- DO NOT REASSIGN / REORDER NIOS_INPUT_INTERNAL & NIOS_OUTPUT_INTERNAL without
	-- updating POWER_INC.H so that the SW and HW remain tied up
	------------------------------------------------------------------------------
	LEDs_Go <= '1' & '1' & '1' & NIOS_Output_Internal(0) & LedFlash_FPGA_Alive(3);
	NIOS_Input_Internal <= "0000000" & POK_Group_4 & POK_Group_3 & POK_Group_2 & POK_Group_1 & User_Switch_5 & User_Switch_4 & User_Switch_3 & User_Switch_2 & User_Switch_1;
	
	Enable_Group_1 <= NIOS_Output_Internal(1);
	Discharge_1 <= NIOS_Output_Internal(2);
	Enable_Group_2 <= NIOS_Output_Internal(3);
	Discharge_2 <= NIOS_Output_Internal(4);
	Enable_Group_3 <= NIOS_Output_Internal(5);
	Discharge_3 <= NIOS_Output_Internal(6);
	Enable_Group_4 <= NIOS_Output_Internal(7);
	Discharge_4 <= NIOS_Output_Internal(8);
	
end RTL;



