/**********************************************************
 * Main code loop for Power Sequencer.
 * Tried to keep the main loop simple and clean.
 * Targeted initially for MAX10 10M08 Evaluation Development board
 * See associated design PDF for detailed description of code but meanwhile:
 *  - Wait for User input from DIP Switch 1 to begin POWERUP sequence
 *  - PowerUp all four power groups in sequence (checking POK lines)
 *  - Wait for POWERDOWN sequence initiation. This could be from:
 *    - User input from DIP Switch 1
 *    - Check any of the four power group POK lines
 *    - Read the analogue value of VIN (if high or low)
 *  - Begin Power Down sequence
 *    - Check User DIP Switch 2 (discharge enable/disable)
 *    - Disable each power group (in reverse order)
 *  - Wait for user toggle of DIP Switch 1 in order to repeat sequence
 *
 *  ADC IP is set to read 5 analague channels and TSD. Only looking at
 *  ADC Channel 1 for this example (VIN)
 *
 *  Look in POWER_INC.H for debug switches and enable/disable delays
 *
 *  LED D2 lights for Power Up and goes off for Power Dn
 *
 *  Code space can be further reduced by removing JTAG UART and
 *  alt_printfs which are only needed for debug
 *
 * Mark Frost (mfrost@altera.com), Altera UK, March 2016
 *********************************************************/

/////////////////////////////////// 
// Includes and function prototypes
/////////////////////////////////// 
#include "system.h"
#include <alt_types.h>
#include <sys/alt_sys_init.h>
#include "altera_avalon_pio_regs.h"
#include "altera_modular_adc.h"
#include "power_inc.h"

int main()
{
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	// GENERAL SETUP AND INITIALISATION
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////

	int count = 0;
	int delay;
	int read_reg, POK, write_reg;
	int powerdown_holdoff = 1;
	int discharge_enable = 1;
	
	alt_u32 *adc_data;
	alt_u32 adc_c1, adc_c2, adc_c3, adc_c4, adc_c5, fpga_temp;
	alt_u32 adc_c0a;
	
#if DEBUG
	alt_printf("Start of general initialisation....\n");
	alt_printf("************ System Information ************\n");
	alt_printf("NIOS_IO_BASE:    0x%08X\n",NIOS_IO_BASE);
	alt_printf("ADC_SAMPLE_STORE_CSR_BASE:    0x%08X\n",ADC_SAMPLE_STORE_CSR_BASE);
	alt_printf("ADC_SEQUENCER_CSR_BASE:       0x%08X\n",ADC_SEQUENCER_CSR_BASE);
	alt_printf("Number of Slots in Sequencer: %d\n\n",ADC_SEQUENCER_CSR_CSD_LENGTH);
	alt_printf("Sequencer Slot 0: %s\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_0);
	alt_printf("Sequencer Slot 1: %s\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_1);
	alt_printf("Sequencer Slot 2: %s\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_2);
	alt_printf("Sequencer Slot 3: %s\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_3);
	alt_printf("Sequencer Slot 4: %s\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_4);
	alt_printf("Sequencer Slot 5: %s\n\n",ADC_SAMPLE_STORE_CSR_CSD_SLOT_5);
	alt_printf("********************************************\n\n");
#endif

 	adc_stop(ADC_SEQUENCER_CSR_BASE);										// ADC setup
	adc_interrupt_disable(ADC_SEQUENCER_CSR_BASE);			
	adc_clear_interrupt_status(ADC_SEQUENCER_CSR_BASE);
	adc_set_mode_run_continuously(ADC_SEQUENCER_CSR_BASE);
	adc_start(ADC_SEQUENCER_CSR_BASE);			
 	write_reg = ALL_OFF | NIOS_OK_LED_OFF;									// All NIOS Outputs to ZERO
	IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);

	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	// MAIN LOOP MAIN LOOP MAIN LOOP MAIN LOOP MAIN LOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOPPPPPPPPPPPPPPPP
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////////////////////////////////
	while(1)
	{
		alt_printf("\nMAX10 Power Sequencing example design\n");
		alt_printf("\n!***** Wait for user input to begin power-up sequence\n");
		
		// Read NIOS input lines check for USER input via Eval board DIP switch #1
		// If switch is on, trigger a Power Up Sequence as defined in the documentation
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & USER_DIP_SWITCH_1) != 1);
		
		///////////////////////////////////////////////////////////////////
		// Enable Power Group 1. Enable device and wait before sampling POK
		///////////////////////////////////////////////////////////////////
		alt_printf("Enabling Mighty Power Group 1\n");
		write_reg = POWERUP1 | NIOS_OK_LED_OFF;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;
		while(delay < POK_DELAY)
		{
			delay++;
		}
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & POK_GROUP_1) != POK_GROUP_1);

		///////////////////////////////////////////////////////////////////
		// Enable Power Group 2. Enable device and wait before sampling POK
		///////////////////////////////////////////////////////////////////
		alt_printf("Enabling Mighty Power Group 2\n");
		write_reg = POWERUP2 | NIOS_OK_LED_OFF;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;
		while(delay < POK_DELAY)
		{
			delay++;
		}
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & POK_GROUP_2) != POK_GROUP_2);

		///////////////////////////////////////////////////////////////////
		// Enable Power Group 3. Enable device and wait before sampling POK
		///////////////////////////////////////////////////////////////////
		alt_printf("Enabling Mighty Power Group 3\n");
		write_reg = POWERUP3 | NIOS_OK_LED_OFF;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;
		while(delay < POK_DELAY)
		{
			delay++;
		}
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & POK_GROUP_3) != POK_GROUP_3);

		///////////////////////////////////////////////////////////////////
		// Enable Power Group 4. Enable device and wait before sampling POK
		///////////////////////////////////////////////////////////////////
		alt_printf("Enabling Mighty Power Group 4\n");
		write_reg = POWERUP4 | NIOS_OK_LED_OFF;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;
		while(delay < POK_DELAY)
		{
			delay++;
		}
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & POK_GROUP_4) != POK_GROUP_4);

		////////////////////////////////////////////////////////////////////
		// In theory all four Power Groups are up and running. Wooooooooo!
		// Now sit and wait for one of the following power down conditions:
		// - !POK
		// - Vout out of spec
		// - User Switch for Power Down
		////////////////////////////////////////////////////////////////////
		alt_printf("All Groups running OK, now wait for PowerDown condition\n\n");
		powerdown_holdoff = 0;
		write_reg = POWERUP4;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);

		while (powerdown_holdoff == 0)
		{
			//////////////////
			// Check POK lines
			//////////////////
			read_reg = IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE);
			if ((read_reg & POK_GROUP_4) != POK_GROUP_4)
			{
				alt_printf("!***** POK4 is unhappy %i\n", read_reg);
				powerdown_holdoff=1;
			}
			else if ((read_reg & POK_GROUP_3) != POK_GROUP_3)
			{
				alt_printf("!***** POK3 is unhappy %i\n", read_reg);
				powerdown_holdoff=1;
			}
			else if ((read_reg & POK_GROUP_2) != POK_GROUP_2)
			{
				alt_printf("!***** POK2 is unhappy %i\n", read_reg);
				powerdown_holdoff=1;
			}
			else if ((read_reg & POK_GROUP_1) != POK_GROUP_1)
			{
				alt_printf("!***** POK1 is unhappy %i\n", read_reg);
				powerdown_holdoff=1;
			}

			///////////////////////////////////////////////////
			// Check USER selected power down via DIP Switch #1
			///////////////////////////////////////////////////
			if ((read_reg & USER_DIP_SWITCH_1) == 0)
			{
				alt_printf("!***** User Enabled PowerDown sequence \n");
				powerdown_holdoff=1;
			}

			///////////////////////////////////////////////////////////////////////////
			// Read from ADC
			// Raw ADC data reports 0 to 4095 representing 0v to 2.5v
			// The signal conditioning on the board scales any input from 0-5v to 0-2v5
			///////////////////////////////////////////////////////////////////////////
 			alt_adc_word_read(ADC_SAMPLE_STORE_CSR_BASE, adc_data, ADC_SEQUENCER_CSR_CSD_LENGTH);
			
			adc_c1 = adc_data[0]; 
			adc_c2 = adc_data[1]; 
			adc_c3 = adc_data[2]; 
			adc_c4 = adc_data[3];

			#if DISPLAY_ADC_MONITOR // DEBUG ONLY
				alt_printf("+-+-+- ADC CH1,2,3,4: %x %x %x %x ***\n", adc_c1, adc_c2, adc_c3, adc_c4);
				delay = 0;
				while(delay < 100000)
				{
					delay++;
				}
			#endif

			if (adc_c1 < VIN_LOW_LIMIT)
			{
				alt_printf("!***** VIN fallen below lower limit \n"); //add vin filter - detect, wait, detect and then fail
				powerdown_holdoff=1;
			} 
			
			if (adc_c1 > VIN_HIGH_LIMIT)
			{
				alt_printf("!***** VIN risen above upper limit \n");
				powerdown_holdoff=1;
			} 
 
		}

		////////////////////////////////////////////////////////////////////
		// Beginning Power Down sequence
		////////////////////////////////////////////////////////////////////
		alt_printf("!***** Powering Down All Groups\n");
		
		/////////////////// EXTRA DEBUG - READ DIP SWITCH 2 and ENABLE/DISABLE DISCHARGE LINES
		if((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & USER_DIP_SWITCH_2) != USER_DIP_SWITCH_2)
			discharge_enable = 1;
		else
		{		
			discharge_enable = 0;
			alt_printf("DISCHARGE DISABLED!\n");
		}

		// Power Down Group 4 - power off device, wait, set discharge line
		alt_printf("Powering down Group 4\n");
		write_reg = POWERDN4;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;			
		while(delay < DISCHARGE_DELAY)
		{
			delay++;
		}
		if (discharge_enable)
		{
			write_reg = DIS4;
			IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		}

		// Power Down Group 3 - power off device, wait, set discharge line
		alt_printf("Powering down Group 3\n");
		write_reg = POWERDN3;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;			
		while(delay < DISCHARGE_DELAY)
		{
			delay++;
		}
		if (discharge_enable)
		{
			write_reg = DIS3;
			IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		}

		// Power Down Group 2 - power off device, wait, set discharge line
		alt_printf("Powering down Group 2\n");
		write_reg = POWERDN2;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;			
		while(delay < DISCHARGE_DELAY)
		{
			delay++;
		}
		if (discharge_enable)
		{
			write_reg = DIS2;
			IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		}

		// Power Down Group 1 - power off device, wait, set discharge line
		alt_printf("Powering down Group 1\n");
		write_reg = POWERDN1;
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		delay = 0;			
		while(delay < DISCHARGE_DELAY)
		{
			delay++;
		}
		if (discharge_enable)
		{
			write_reg = DIS1;
			IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);
		}

		// Finally, wait for a while and then disable all active outputs
		delay = 0;			
		while(delay < END_DELAY)
		{
			delay++;
		}
		write_reg = ALL_OFF | NIOS_OK_LED_OFF;											// All NIOS Outputs to ZERO (and LED OFF)
		IOWR_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE, write_reg);

		/////////////////////////////////////////////////////////////////
		// Toggle User DIP Switch #1 off/on to restart the sequence again
		/////////////////////////////////////////////////////////////////
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & USER_DIP_SWITCH_1) != 1);
		while ((IORD_ALTERA_AVALON_PIO_DATA(NIOS_IO_BASE) & USER_DIP_SWITCH_1) != 0);
		
	}
}
